## 配置模块使用说明

配置模块提供了统一的配置管理接口，主要功能包括：

- 获取配置实例
- 获取配置项值
- 注册配置变更回调

### 获取配置实例

```python
from app.config import get_config

config = get_config()
```

### 获取配置项值

```python
from app.config import get_config_value

# 获取配置项值，如果不存在则返回默认值
value = get_config_value('key', default='default_value')
```

### 注册配置变更回调

```python
from app.config import on_config_change

def callback():
    print('配置已变更')

on_config_change(callback)
```

### 异常处理

所有接口都可能会抛出`RuntimeError`异常，建议在使用时进行捕获处理：

```python
try:
    config = get_config()
except RuntimeError as e:
    print(f'获取配置失败：{e}')
```

## 环境配置说明

项目支持多环境配置，通过不同的配置文件管理：

- application.yml：基础配置
- application-dev.yml：开发环境配置
- application-prod.yml：生产环境配置

### 配置优先级

1. 环境特定配置（application-{env}.yml）
2. 基础配置（application.yml）

### 环境切换

通过设置环境变量`APP_ENV`来切换环境：

#### 项目级设置（推荐）

在项目根目录下创建`.env`文件，内容如下：

```env
# 开发环境
APP_ENV=dev

# 生产环境
# APP_ENV=prod
```

注意：

1. 该设置仅对当前项目有效
2. .env 文件应添加到.gitignore 中，避免将环境配置提交到代码库
3. 如果同时存在系统环境变量和.env 文件，.env 文件的优先级更高

#### Windows 系统

1. 临时设置（仅当前命令行窗口有效）：

```cmd
:: 开发环境
set APP_ENV=dev

:: 生产环境
set APP_ENV=prod
```

2. 永久设置：

- 右键点击"此电脑" -> "属性" -> "高级系统设置"
- 点击"环境变量"按钮
- 在"系统变量"或"用户变量"中新建变量：
  - 变量名：APP_ENV
  - 变量值：dev（开发环境）或 prod（生产环境）

#### macOS/Linux 系统

```bash
# 开发环境
export APP_ENV=dev

# 生产环境
export APP_ENV=prod
```

### 各环境主要差异

| 配置项               | 开发环境          | 生产环境            |
| -------------------- | ----------------- | ------------------- |
| app.debug            | true              | false               |
| database.host        | localhost         | db.prod.example.com |
| database.name        | question_bank_dev | question_bank_prod  |
| logging.path         | logs              | /var/log/app        |
| logging.level        | DEBUG             | INFO                |
| logging.retention    | 7 天              | 30 天               |
| logging.max_size     | 10MB              | 100MB               |
| logging.backup_count | 5                 | 30                  |

## 日志配置说明

项目使用 loguru 库进行日志管理，主要特性包括：

- 开发环境：

  - 日志级别：DEBUG
  - 日志路径：logs/
  - 日志保留：7 天
  - 日志格式：包含时间、级别、文件名和行号
  - 控制台输出：彩色格式

- 生产环境：
  - 日志级别：INFO
  - 日志路径：/var/log/app
  - 日志保留：30 天
  - 日志格式：仅包含时间、级别和消息
  - 控制台输出：简洁格式

使用示例：

```python
from app.logger import logger

logger.info("这是一条信息日志")
logger.error("这是一条错误日志", exc_info=True)
```
